package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.springframework.util.ResourceUtils</code>.
 * 
 * @author Rein Raudjärv
 */
public class ResourceUtilsCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    /*
     * Replace
     * 
     * if (!"file".equals(resourceUrl.getProtocol()))
     *   throw new FileNotFoundException();
     *
     * with
     *
     * if (!"file".equals(resourceUrl.getProtocol()) && !"vfsfile".equals(resourceUrl.getProtocol()))
     *   throw new FileNotFoundException();
     */

    ctClass.getDeclaredMethod("getFile",
        cp.get(new String[] { "java.net.URL", "java.lang.String" })).instrument(new ExprEditor() {
          public void edit(MethodCall m) throws CannotCompileException {
            if ("java.lang.String".equals(m.getClassName())
                && "equals".equals(m.getMethodName())) {
              m.replace(
                  "{" +
                  "  boolean result = $proceed($$);" +
                  "  if (\"file\".equals($0) && \"vfsfile\".equals($1))" +
                  "    result = true;" +
                  "  $_ = result;" +
                  "}");
            }
          }
        });
  }
}
